// This code is run during the subset 1 autotests. Do not modify it!

#ifdef TEST_SUBSET_1

#include <stdint.h>
#include <stdio.h>
#include <inttypes.h>
#include <string.h>
#include <stdlib.h>


struct imps_file {
    uint32_t num_instructions;
    uint32_t entry_point;
    uint32_t *instructions;
    uint32_t *debug_offsets;
    uint16_t memory_size;
    uint8_t *initial_data;
};

void read_imps_file(char *path, struct imps_file *executable);


static void dump_imps_file(struct imps_file *executable) {
    printf("executable:\n");
    printf("  num_instructions: %" PRIu32 "\n", executable->num_instructions);
    printf("  entry_point: %" PRIu32 "\n", executable->entry_point);

    for (size_t i = 0; i < executable->num_instructions; i++) {
        printf(
            "  instructions[%zu]: 0x%08" PRIx32 "\n",
            i, executable->instructions[i]
        );
    }

    for (size_t i = 0; i < executable->num_instructions; i++) {
        printf(
            "  debug_offsets[%zu]: %" PRIu32 "\n",
            i, executable->debug_offsets[i]
        );
    }

    for (size_t i = 0; i < executable->memory_size; i++) {
        printf(
            "  initial_memory_contents[%zu]: %" PRIx8 "\n",
            i, executable->initial_data[i]
        );
    }
}

#ifdef main
#undef main
#endif

int main(int argc, char *argv[]) {
    if (argc != 2) {
        fprintf(stderr, "Usage: imps <executable>\n");
    }

    struct imps_file executable;
    memset(&executable, 0, sizeof(executable));
    read_imps_file(argv[1], &executable);

    dump_imps_file(&executable);

    if (executable.debug_offsets != NULL) {
        free(executable.debug_offsets);
    }
    if (executable.instructions != NULL) {
        free(executable.instructions);
    }
}

#endif
